import { Response } from 'express';
import { chartMapper } from './chart';
import { isAdmin } from '../../middleware/is-admin';
import { RequestContext } from '../../util/request-context.type';
import { Chart } from '@baseline/types/chart';
import { getErrorMessage } from '../../util/error-message';
import createApp from '../../util/express-app';
import createAuthenticatedHandler from '../../util/create-authenticated-handler';
import { chartService } from './chart.service';

const app = createApp();
export const handler = createAuthenticatedHandler(app);

app.post('/chart/admin', [
  isAdmin,
  async (req: RequestContext, res: Response) => {
    try {
      const { md5, sha256, resourceUri, parentChart, name, comment } =
        req.body as Chart;
      const chartData: Partial<Chart> = {
        md5,
        sha256,
        resourceUri,
        parentChart,
        name,
        comment,
      };
      const chart = await chartService.create(chartData);
      res.json(chartMapper(chart));
    } catch (error) {
      const message = getErrorMessage(error);
      console.error(`Failed to create chart ${message}`);
      res.status(400).json({ error: 'Failed to create chart' });
    }
  },
]);

app.post('/chart/admin/bulk', [
  isAdmin,
  async (req: RequestContext, res: Response) => {
    try {
      const chartData = (req.body as Chart[]).map((chart) => ({
        md5: chart.md5,
        sha256: chart.sha256,
        resourceUri: chart.resourceUri,
        parentChart: chart.parentChart,
        name: chart.name,
        comment: chart.comment,
      }));

      const charts = await Promise.all(
        chartData.map((chart) => chartService.create(chart)),
      );

      res.json(charts.map(chartMapper));
    } catch (error) {
      const message = getErrorMessage(error);
      console.error(`Failed to create chart ${message}`);
      res.status(400).json({ error: 'Failed to create chart' });
    }
  },
]);

app.patch('/chart/admin', [
  isAdmin,
  async (req: RequestContext, res: Response) => {
    try {
      const { chartId, md5, sha256, resourceUri, parentChart, name, comment } =
        req.body as Chart;
      const chartData: Partial<Chart> = {
        chartId,
        md5,
        sha256,
        resourceUri,
        parentChart,
        name,
        comment,
      };
      const chart = await chartService.update(chartData);
      res.json(chartMapper(chart));
    } catch (error) {
      const message = getErrorMessage(error);
      console.error(`Failed to update chart: ${message}`);
      res.status(400).json({
        error: 'Failed to update chart',
      });
    }
  },
]);

app.delete('/chart/admin/:chartId', [
  isAdmin,
  async (req: RequestContext, res: Response) => {
    try {
      const chartId = req.params.chartId;
      await chartService.delete(chartId);
      res.status(200);
      res.send();
    } catch (error) {
      const message = getErrorMessage(error);
      console.error(`Failed to delete chart: ${message}`);
      res.status(400).json({
        error: 'Failed to delete chart',
      });
    }
  },
]);

app.get('/chart/admin/list', [
  isAdmin,
  async (req: RequestContext, res: Response) => {
    try {
      const charts = await chartService.getAll();
      const formattedCharts = charts.map(chartMapper);
      res.json(formattedCharts);
    } catch (error) {
      const message = getErrorMessage(error);
      console.error(`Failed to get charts: ${message}`);
      res.status(400).json({
        error: 'Failed to get charts',
      });
    }
  },
]);

app.get('/chart/admin/:chartId', [
  isAdmin,
  async (req: RequestContext, res: Response) => {
    try {
      const chart = await chartService.get(req.params.chartId);
      res.json(chartMapper(chart));
    } catch (error) {
      const message = getErrorMessage(error);
      console.error(`Failed to get chart: ${message}`);
      res.status(400).json({
        error: 'Failed to get chart',
      });
    }
  },
]);
